@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2004  Motoyuki Kasahara
+ * Copyright (c) 2003-2006  Motoyuki Kasahara
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -431,6 +431,14 @@ ebnet_get_addresses(const char *host, char *ipv6_address, char *ipv4_address)
 	    ipv6_address, INET6_ADDRSTRLEN + IF_NAMESIZE,
 	    dummy_service, sizeof(dummy_service),
 	    NI_NUMERICHOST | NI_NUMERICSERV | NI_WITHSCOPEID);
+#if NI_WITHSCOPEID != 0
+	if (gai_error != 0) {
+	    gai_error = getnameinfo(info->ai_addr, info->ai_addrlen,
+		ipv6_address, INET6_ADDRSTRLEN + IF_NAMESIZE,
+		dummy_service, sizeof(dummy_service),
+		NI_NUMERICHOST | NI_NUMERICSERV);
+	}
+#endif
 	if (gai_error == 0)
 	    break;
 	*ipv6_address = '\0';
@@ -539,6 +547,9 @@ ebnet_create_new_connection(const char *address, int port)
     int new_file = -1;
     int gai_error;
     char port_string[IN_PORTSTRLEN];
+#ifdef O_NONBLOCK
+    int file_flags;
+#endif
 
     if (port < 0 || 65535 < port)
 	goto failed;
@@ -563,6 +574,12 @@ ebnet_create_new_connection(const char *address, int port)
     if (connect(new_file, info_list->ai_addr, info_list->ai_addrlen) < 0)
 	goto failed;
 
+#ifdef O_NONBLOCK
+    file_flags = fcntl(new_file, F_GETFL, 0);
+    if (file_flags >= 0)
+	fcntl(new_file, F_SETFL, file_flags | O_NONBLOCK);
+#endif
+
     freeaddrinfo(info_list);
 
     return new_file;
